@@ -4,7 +4,7 @@
     figures: [],
     plugin_map: {}
   };
-  mpld3.version = "0.2git";
+  mpld3.version = "0.3";
   mpld3.register_plugin = function(name, obj) {
     mpld3.plugin_map[name] = obj;
   };
@@ -327,7 +327,18 @@
     return new mpld3_Grid(this.ax, gridprop);
   };
   mpld3_Axis.prototype.draw = function() {
-    this.axis = d3.svg.axis().scale(this.scale).orient(this.props.position).ticks(this.props.nticks).tickValues(this.props.tickvalues).tickFormat(this.props.tickformat);
+    var scale = this.props.xy === "x" ? this.parent.props.xscale : this.parent.props.yscale;
+    if (scale === "date" && this.props.tickvalues) {
+      var domain = this.props.xy === "x" ? this.parent.x.domain() : this.parent.y.domain();
+      var range = this.props.xy === "x" ? this.parent.xdom.domain() : this.parent.ydom.domain();
+      var ordinal_to_js_date = d3.scale.linear().domain(domain).range(range);
+      this.props.tickvalues = this.props.tickvalues.map(function(value) {
+        return new Date(ordinal_to_js_date(value));
+      });
+    }
+    var tickformat = mpld3_tickFormat(this.props.tickformat, this.props.tickvalues);
+    this.axis = d3.svg.axis().scale(this.scale).orient(this.props.position).ticks(this.props.nticks).tickValues(this.props.tickvalues).tickFormat(tickformat);
+    this.filter_ticks(this.axis.tickValues, this.axis.scale().domain());
     this.elem = this.ax.baseaxes.append("g").attr("transform", this.transform).attr("class", this.cssclass).call(this.axis);
     mpld3.insert_css("div#" + this.ax.fig.figid + " ." + this.cssclass + " line, " + " ." + this.cssclass + " path", {
       "shape-rendering": "crispEdges",
@@ -336,14 +347,29 @@
     });
     mpld3.insert_css("div#" + this.ax.fig.figid + " ." + this.cssclass + " text", {
       "font-family": "sans-serif",
-      "font-size": this.props.fontsize,
+      "font-size": this.props.fontsize + "px",
       fill: this.props.fontcolor,
       stroke: "none"
     });
   };
+  function mpld3_tickFormat(tickformat, tickvalues) {
+    if (tickformat === "" || tickformat === null) {
+      return tickformat;
+    } else {
+      return d3.scale.threshold().domain(tickvalues.slice(1)).range(tickformat);
+    }
+  }
   mpld3_Axis.prototype.zoomed = function() {
+    this.filter_ticks(this.axis.tickValues, this.axis.scale().domain());
     this.elem.call(this.axis);
   };
+  mpld3_Axis.prototype.filter_ticks = function(tickValues, domain) {
+    if (this.props.tickvalues != null) {
+      tickValues(this.props.tickvalues.filter(function(v) {
+        return v >= domain[0] && v <= domain[1];
+      }));
+    }
+  };
   mpld3.Coordinates = mpld3_Coordinates;
   function mpld3_Coordinates(trans, ax) {
     if (typeof ax === "undefined") {
@@ -399,7 +425,7 @@
     facecolor: "green",
     edgecolor: "black",
     edgewidth: 1,
-    dasharray: "10,0",
+    dasharray: "none",
     pathcodes: null,
     offset: null,
     offsetcoordinates: "data",
@@ -414,11 +440,11 @@
     this.offsetcoords = new mpld3_Coordinates(this.props.offsetcoordinates, this.ax);
     this.datafunc = mpld3_path();
   }
-  mpld3_Path.prototype.nanFilter = function(d, i) {
-    return !isNaN(d[this.props.xindex]) && !isNaN(d[this.props.yindex]);
+  mpld3_Path.prototype.finiteFilter = function(d, i) {
+    return isFinite(this.pathcoords.x(d[this.props.xindex])) && isFinite(this.pathcoords.y(d[this.props.yindex]));
   };
   mpld3_Path.prototype.draw = function() {
-    this.datafunc.defined(this.nanFilter.bind(this)).x(function(d) {
+    this.datafunc.defined(this.finiteFilter.bind(this)).x(function(d) {
       return this.pathcoords.x(d[this.props.xindex]);
     }).y(function(d) {
       return this.pathcoords.y(d[this.props.yindex]);
@@ -505,9 +531,16 @@
     }
     return ret;
   };
+  mpld3_PathCollection.prototype.allFinite = function(d) {
+    if (d instanceof Array) {
+      return d.length == d.filter(isFinite).length;
+    } else {
+      return true;
+    }
+  };
   mpld3_PathCollection.prototype.draw = function() {
     this.group = this.ax.axes.append("svg:g");
-    this.pathsobj = this.group.selectAll("paths").data(this.offsets).enter().append("svg:path").attr("d", this.pathFunc.bind(this)).attr("class", "mpld3-path").attr("transform", this.transformFunc.bind(this)).attr("style", this.styleFunc.bind(this)).attr("vector-effect", "non-scaling-stroke");
+    this.pathsobj = this.group.selectAll("paths").data(this.offsets.filter(this.allFinite)).enter().append("svg:path").attr("d", this.pathFunc.bind(this)).attr("class", "mpld3-path").attr("transform", this.transformFunc.bind(this)).attr("style", this.styleFunc.bind(this)).attr("vector-effect", "non-scaling-stroke");
   };
   mpld3_PathCollection.prototype.elements = function(d) {
     return this.group.selectAll("path");
@@ -530,9 +563,10 @@
     coordinates: "data",
     color: "salmon",
     linewidth: 2,
-    dasharray: "10,0",
+    dasharray: "none",
     alpha: 1,
-    zorder: 2
+    zorder: 2,
+    drawstyle: "none"
   };
   function mpld3_Line(ax, props) {
     mpld3_PlotElement.call(this, ax, props);
@@ -542,9 +576,27 @@
     delete pathProps.color;
     pathProps.edgewidth = pathProps.linewidth;
     delete pathProps.linewidth;
+    drawstyle = pathProps.drawstyle;
+    delete pathProps.drawstyle;
     this.defaultProps = mpld3_Path.prototype.defaultProps;
     mpld3_Path.call(this, ax, pathProps);
-    this.datafunc = d3.svg.line().interpolate("linear");
+    switch (drawstyle) {
+     case "steps":
+     case "steps-pre":
+      this.datafunc = d3.svg.line().interpolate("step-before");
+      break;
+
+     case "steps-post":
+      this.datafunc = d3.svg.line().interpolate("step-after");
+      break;
+
+     case "steps-mid":
+      this.datafunc = d3.svg.line().interpolate("step");
+      break;
+
+     default:
+      this.datafunc = d3.svg.line().interpolate("linear");
+    }
   }
   mpld3.Markers = mpld3_Markers;
   mpld3_Markers.prototype = Object.create(mpld3_PathCollection.prototype);
@@ -716,7 +768,7 @@
       this.x = mpld3.multiscale(d3.scale.linear().domain(this.props.xlim).range(this.props.xdomain.map(Number)), this.xdom);
     }
     if (this.props.yscale === "date") {
-      this.x = mpld3.multiscale(d3.scale.linear().domain(this.props.ylim).range(this.props.ydomain.map(Number)), this.ydom);
+      this.y = mpld3.multiscale(d3.scale.linear().domain(this.props.ylim).range(this.props.ydomain.map(Number)), this.ydom);
     }
     var axes = this.props.axes;
     for (var i = 0; i < axes.length; i++) {
